from wexample_wex_addon_default.const.types import (UPGRADE_TYPE_MINOR)
from wexample_wex_addon_default.helpers.version import version_increment


def default__version__increment(
    kernel: "Kernel",
    version: str,
    type: str = UPGRADE_TYPE_MINOR,
    increment: int = 1,
    build: bool = False,
) -> str:
    return version_increment(
        version=version,
        type=type,
        increment=increment,
        build=build,
    )



from typing import Any, Dict, List

import click
from click.core import Command
from click.types import BoolParamType
from wexample_helpers.helpers.args import args_convert_dict_to_snake_dict
from wexample_helpers.helpers.string import string_to_kebab_case

from wexample_wex_core.const.types import (
    CoreCommandArgsDict,
    CoreCommandArgsList,
)


def click_args_convert_dict_to_long_names_dict(
    function: Command, args: CoreCommandArgsDict
) -> Dict[str, Any]:
    short_names = {}

    for param in function.params:
        for opt in param.opts:
            # This is a short name
            if opt.startswith("-") and opt[1:2] != "-":
                if param.name:
                    short_names[opt[1:]] = string_to_kebab_case(param.name)

    # Transform short named args to long named args.
    args_long = {}
    for name in args:
        if name in short_names:
            args_long[short_names[name]] = args[name]
        else:
            args_long[name] = args[name]

    return args_long


def click_args_convert_to_dict(function: Command, arg_list: List[str]) -> Dict[str, Any]:
    args_dict: Dict[str, str | bool] = {}

    param_dict = {
        opt.lstrip("-"): param
        for param in function.params
        if isinstance(param, click.Option)
        for opt in param.opts
    }

    i = 0
    while i < len(arg_list):
        arg = arg_list[i]

        if isinstance(arg, str):
            stripped_arg = arg.lstrip("-")

            # Manage parameters defined in function
            if stripped_arg in param_dict:
                param = param_dict[stripped_arg]
                if isinstance(param.type, BoolParamType) or param.is_flag:
                    args_dict[stripped_arg] = True
                    i += 1
                else:
                    i += 1
                    value = arg_list[i]
                    args_dict[stripped_arg] = value
                    i += 1
            # Manage unknown parameters
            else:
                key = arg.lstrip("-")

                if len(arg_list) > i + 1 and arg_list[i + 1][0:1] != "-":
                    args_dict[key] = arg_list[i + 1]
                    i += 1
                else:
                    args_dict[key] = True

                i += 1
        else:
            i += 1

    return args_dict


def click_args_convert_dict_to_args(
    function: Command, args: CoreCommandArgsDict
) -> CoreCommandArgsList:
    """
    Convert args {"my-arg": "value"} to list ["--my_arg", "value"].
    Any key in `args` that is not found in `function.params` is added to the
    argument list as a key-value pair.
    """
    arg_list = []
    args_long = click_args_convert_dict_to_long_names_dict(function=function, args=args)
    args_long = args_convert_dict_to_snake_dict(input_dict=args_long)

    for param in function.params:
        if param.name in args_long:
            if isinstance(param, click.Option):
                param_name_kebab = string_to_kebab_case(param.name)

                if param.is_flag:
                    if args_long[param.name]:
                        arg_list.append(f"--{param_name_kebab}")
                    # Flag passed to False is just removed
                elif args_long[param.name] is not None:
                    arg_list.append(f"--{param_name_kebab}")
                    value = args_long[param.name]
                    if not isinstance(args_long[param.name], bool):
                        value = str(value)
                    arg_list.append(value)
    # Append any remaining arguments as key-value pairs
    for key, value in args_long.items():
        if key not in [param.name for param in function.params] and value is not None:
            arg_list.append(f"--{key}")
            if not isinstance(value, bool):
                # Convert to str to allow joining array.
                arg_list.append(str(value))

    return arg_list



from typing import Mapping, TypedDict, Optional

from wexample_helpers.const.types import StringsList, BasicInlineValue

# Core command types
CoreCommandArgsDict = Mapping[str, BasicInlineValue]
CoreCommandArgsList = StringsList

# Version types
UPGRADE_TYPE_MAJOR: str = "major"
UPGRADE_TYPE_INTERMEDIATE: str = "intermediate"
UPGRADE_TYPE_MINOR: str = "minor"
UPGRADE_TYPE_ALPHA: str = "alpha"
UPGRADE_TYPE_BETA: str = "beta"
UPGRADE_TYPE_DEV: str = "dev"
UPGRADE_TYPE_RC: str = "rc"
UPGRADE_TYPE_NIGHTLY: str = "nightly"
UPGRADE_TYPE_SNAPSHOT: str = "snapshot"

VERSION_PRE_BUILD_NUMBER: int = 0


class VersionDescriptor(TypedDict):
    major: Optional[int]
    intermediate: Optional[int]
    minor: Optional[int]
    pre_build_type: Optional[str]
    pre_build_number: Optional[int]



from typing import Type, Any

from wexample_filestate.file_state_manager import FileStateManager
from wexample_app.utils.abstract_kernel import AbstractKernel
from wexample_wex_core.common.file.kernel_directory_structure import KernelDirectoryStructure
from wexample_app.utils.mixins.command_line_kernel import CommandLineKernel


class Kernel(AbstractKernel, CommandLineKernel):
    def __init__(self, /, **data: Any) -> None:
        super().__init__(**data)
        self._init_command_line_kernel()

    def _get_workdir_state_manager_class(self) -> Type[FileStateManager]:
        return KernelDirectoryStructure



import re
from typing import Optional

from wexample_helpers.const.types import StringsList
from wexample_wex_core.const.types import (
    VersionDescriptor, UPGRADE_TYPE_MINOR, UPGRADE_TYPE_MAJOR,
    UPGRADE_TYPE_INTERMEDIATE, UPGRADE_TYPE_ALPHA, UPGRADE_TYPE_BETA,
    UPGRADE_TYPE_DEV, UPGRADE_TYPE_RC, UPGRADE_TYPE_NIGHTLY,
    UPGRADE_TYPE_SNAPSHOT
)


def is_greater_than(
    first: VersionDescriptor, second: VersionDescriptor, true_if_equal: bool = False
) -> bool:
    keys_to_check: StringsList = [
        "major",
        "intermediate",
        "minor",
        "pre_build_type",
        "pre_build_number",
    ]

    for key in keys_to_check:
        first_value = first.get(key, None)
        second_value = second.get(key, None)

        if first_value is not None and second_value is None:
            return False
        elif first_value is None and second_value is not None:
            return True

        if first_value is not None and second_value is not None:
            assert isinstance(first_value, int)
            assert isinstance(second_value, int)
            if first_value < second_value:
                return False
            elif first_value > second_value:
                return True

    return true_if_equal


def version_join(version: VersionDescriptor, add_build: bool | str = False) -> str:
    output = f"{version['major']}.{version['intermediate']}.{version['minor']}"

    # Build version string
    if version["pre_build_type"]:
        output += f'-{version["pre_build_type"]}.{version["pre_build_number"]}'

    if add_build:
        if isinstance(add_build, str):
            output += f"+build.{add_build}"
        else:
            import datetime
            output += f"+build." + datetime.datetime.now().strftime("%Y%m%d%H%M%S")

    return output


def version_parse(version: str) -> VersionDescriptor | None:
    pre_build_number: Optional[int] = None
    pre_build_type: Optional[str] = None

    try:
        # Handle 1.0.0-beta.1+build.1234
        if "-" in version:
            base_version, pre_build = version.split("-")

            if "." in pre_build:
                pre_build_parts = pre_build.split(".")
                pre_build_type = pre_build_parts[0]

                # pre_build_number can be : 1+build.1234
                if "+" in pre_build_parts[1]:
                    pre_build_number_str, build_metadata = pre_build_parts[1].split("+")
                    pre_build_number = (
                        int(pre_build_number_str) if pre_build_number_str else None
                    )
                else:
                    pre_build_number = (
                        int(pre_build_parts[1]) if pre_build_parts[1] else None
                    )

        base_parts = version.split("-")[0].split(".")
        for part in base_parts:
            if part and not part.isdigit():
                return None

        match = re.match(r"(\d+)?\.?(\d+)?\.?(\d+)?([-.+].*)?", version)
        if not match:
            return None
            
        major, intermediate, minor, _ = match.groups()
        
        if not (major or intermediate or minor):
            return None

        # Create a dictionary to store the elements
        version_dict: VersionDescriptor = {
            "major": int(major) if major else None,
            "intermediate": int(intermediate) if intermediate else None,
            "minor": int(minor) if minor else None,
            "pre_build_type": pre_build_type,
            "pre_build_number": pre_build_number,
        }
    except Exception:
        return None

    return version_dict


def version_increment(
    version: str,
    type: str = UPGRADE_TYPE_MINOR,
    increment: int = 1,
    build: bool | str = False,
) -> str:
    version_dict = version_parse(version)
    if not version_dict:
        raise ValueError(f"Invalid version format: {version}")

    # Increment according to type
    if type == UPGRADE_TYPE_MAJOR:
        version_dict["major"] = int(version_dict["major"]) + increment
        version_dict["intermediate"] = 0
        version_dict["minor"] = 0
    elif type == UPGRADE_TYPE_INTERMEDIATE:
        version_dict["intermediate"] = int(version_dict["intermediate"]) + increment
        version_dict["minor"] = 0
    elif type == UPGRADE_TYPE_MINOR:
        version_dict["minor"] = int(version_dict["minor"]) + increment
    # Any of pre-build version
    elif type in [
        UPGRADE_TYPE_ALPHA,
        UPGRADE_TYPE_BETA,
        UPGRADE_TYPE_DEV,
        UPGRADE_TYPE_RC,
        UPGRADE_TYPE_NIGHTLY,
        UPGRADE_TYPE_SNAPSHOT,
    ]:
        version_dict["pre_build_type"] = type
        version_dict["pre_build_number"] = increment

    return version_join(version_dict, build)



from typing import TYPE_CHECKING

from pydantic import BaseModel

if TYPE_CHECKING:
    from wexample_wex_core.common.kernel import Kernel


class AbstractKernelChild(BaseModel):
    kernel: "Kernel"

    def __init__(self, kernel: "Kernel") -> None:
        super().__init__()
        self.kernel = kernel
